#!/usr/bin/python
"""Testing setting locks on locks """

from RECT import *


class SingleSetLockOnLock(TestCase):
    """Set lock on a lock tests with one slave (thread).

    Provided tests list:

    1. Set a read lock on a read lock (Allowed)
    2. Set a write lock on a write lock (Denied)
    3. Set a write lock on a read lock (Denied)
    4. Set a read lock on a write lock (Denied)

    Tests independed of a locking type (advisory or mandatory)
    """
    n_shares = 1
    n_slaves = 1

    def setUp(self):
        TestCase.setUp(self)
        self.filename = "%s/%s" % (self.shares[0],
                                   self.config.get('filename', 'test.dat'))

    def testSetReadLockOnReadLock(self):
        """Set a read lock on a read lock (Allowed)

        Slave0: Create a file, reopen it for reading, set a read lock
        Slave0: Open file for reading, set a read lock and close file
        Slave0: close the file and remove it.
        """

        slave = self.slaves[0]
        slave.mark(self.id())
        slave.mount(self.shares[0])
        file = slave.open(self.filename, OpenMode.WRITE, CreationMode.CREATE)
        file.close()

        file1 = slave.open(self.filename, OpenMode.READ, CreationMode.OPEN)
        file1.setLock(Lock(0, 500, LockType.LOCKREAD))

        file2 = slave.open(self.filename, OpenMode.READ, CreationMode.OPEN)
        file2.setLock(Lock(0, 500, LockType.LOCKREAD))

        file2.close()
        file1.close()
        slave.unlink(self.filename)
        slave.umount(self.shares[0])


    def testSetWriteLockOnWriteLock(self):
        """Set a write lock on a write lock (Denied)

        Slave0: Create a file, set a write lock
        Slave0: Open file for writing, try to set a write lock and close file
        Slave0: close the file and remove it.
        """

        slave = self.slaves[0]
        slave.mark(self.id())
        slave.mount(self.shares[0])
        file1 = slave.open(self.filename, OpenMode.WRITE, CreationMode.CREATE)
        file1.setLock(Lock(0, 500, LockType.LOCKWRITE))

        file2 = slave.open(self.filename, OpenMode.WRITE, CreationMode.OPEN)
        self.assertRaises(RECT.Error, file2.setLock,
                          Lock(0, 500, LockType.LOCKWRITE))

        file2.close()
        file1.close()
        slave.unlink(self.filename)
        slave.umount(self.shares[0])


    def testSetWriteLockOnReadLock(self):
        """Set a write lock on a read lock (Denied)

        Slave0: Create a file, reopen it for reading, set a read lock
        Slave0: Open file for writing, try to set a write lock and close file
        Slave0: close the file and remove it.
        """

        slave = self.slaves[0]
        slave.mark(self.id())
        slave.mount(self.shares[0])
        file = slave.open(self.filename, OpenMode.WRITE, CreationMode.CREATE)
        file.close()

        file1 = slave.open(self.filename, OpenMode.READ, CreationMode.OPEN)
        file1.setLock(Lock(0, 500, LockType.LOCKREAD))

        file2 = slave.open(self.filename, OpenMode.WRITE, CreationMode.OPEN)
        self.assertRaises(RECT.Error, file2.setLock,
                          Lock(0, 500, LockType.LOCKWRITE))

        file2.close()
        file1.close()
        slave.unlink(self.filename)
        slave.umount(self.shares[0])


    def testSetReadLockOnWriteLock(self):
        """Set a read lock on a write lock (Denied)

        Slave0: Create a file, set a write lock
        Slave0: Open file for reading, try to set a read lock and close file
        Slave0: close the file and remove it.
        """

        slave = self.slaves[0]
        slave.mark(self.id())
        slave.mount(self.shares[0])
        file1 = slave.open(self.filename, OpenMode.WRITE, CreationMode.CREATE)
        file1.setLock(Lock(0, 500, LockType.LOCKWRITE))

        file2 = slave.open(self.filename, OpenMode.READ, CreationMode.OPEN)
        self.assertRaises(RECT.Error, file2.setLock,
                          Lock(0, 500, LockType.LOCKREAD))

        file2.close()
        file1.close()
        slave.unlink(self.filename)
        slave.umount(self.shares[0])


class DualSetLockOnLock(TestCase):
    """Set lock on a lock tests with two slaves (process).

    Provided tests list:

    1. Set a read lock on a read lock (Allowed)
    2. Set a write lock on a write lock (Denied)
    3. Set a write lock on a read lock (Denied)
    4. Set a read lock on a write lock (Denied)

    Tests independed of a locking type (advisory or mandatory)
    """
    n_shares = 1
    n_slaves = 2

    def setUp(self):
        TestCase.setUp(self)
        self.filename = "%s/%s" % (self.shares[0],
                                   self.config.get('filename', 'test.dat'))

    def testSetReadLockOnReadLock(self):
        """Set a read lock on a read lock (Allowed)

        Slave0: Create a file, reopen it for reading, set a read lock
        Slave1: Open file for reading, set a read lock and close file
        Slave0: close the file and remove it.
        """

        self.slaves[0].mark(self.id())
        self.slaves[0].mount(self.shares[0])
        file = self.slaves[0].open(self.filename, OpenMode.WRITE,
                                   CreationMode.CREATE)
        file.close()

        file1 = self.slaves[0].open(self.filename, OpenMode.READ,
                                    CreationMode.OPEN)
        file1.setLock(Lock(0, 500, LockType.LOCKREAD))

        self.slaves[1].mark(self.id())
        self.slaves[1].mount(self.shares[0])
        file2 = self.slaves[1].open(self.filename, OpenMode.READ,
                                    CreationMode.OPEN)
        file2.setLock(Lock(0, 500, LockType.LOCKREAD))

        file2.close()
        file1.close()
        self.slaves[0].unlink(self.filename)
        self.slaves[1].umount(self.shares[0])
        self.slaves[0].umount(self.shares[0])


    def testSetWriteLockOnWriteLock(self):
        """Set a write lock on a write lock (Denied)

        Slave0: Create a file, set a write lock
        Slave1: Open file for writing, try to set a write lock and close file
        Slave0: close the file and remove it.
        """

        self.slaves[0].mark(self.id())
        self.slaves[0].mount(self.shares[0])
        file1 = self.slaves[0].open(self.filename, OpenMode.WRITE,
                                    CreationMode.CREATE)
        file1.setLock(Lock(0, 500, LockType.LOCKWRITE))

        self.slaves[1].mark(self.id())
        self.slaves[1].mount(self.shares[0])
        file2 = self.slaves[1].open(self.filename, OpenMode.WRITE,
                                    CreationMode.OPEN)
        self.assertRaises(RECT.Error, file2.setLock,
                          Lock(0, 500, LockType.LOCKWRITE))

        file2.close()
        file1.close()
        self.slaves[0].unlink(self.filename)
        self.slaves[1].umount(self.shares[0])
        self.slaves[0].umount(self.shares[0])


    def testSetWriteLockOnReadLock(self):
        """Set a write lock on a read lock (Denied)

        Slave0: Create a file, reopen it for reading, set a read lock
        Slave1: Open file for writing, try to set a write lock and close file
        Slave0: close the file and remove it.
        """

        self.slaves[0].mark(self.id())
        self.slaves[0].mount(self.shares[0])
        file = self.slaves[0].open(self.filename, OpenMode.WRITE,
                                   CreationMode.CREATE)
        file.close()

        file1 = self.slaves[0].open(self.filename, OpenMode.READ,
                                    CreationMode.OPEN)
        file1.setLock(Lock(0, 500, LockType.LOCKREAD))

        self.slaves[1].mark(self.id())
        self.slaves[1].mount(self.shares[0])
        file2 = self.slaves[1].open(self.filename, OpenMode.WRITE,
                                    CreationMode.OPEN)
        self.assertRaises(RECT.Error, file2.setLock,
                          Lock(0, 500, LockType.LOCKWRITE))

        file2.close()
        file1.close()
        self.slaves[0].unlink(self.filename)
        self.slaves[1].umount(self.shares[0])
        self.slaves[0].umount(self.shares[0])


    def testSetReadLockOnWriteLock(self):
        """Set a read lock on a write lock (Denied)

        Slave0: Create a file, set a write lock
        Slave1: Open file for reading, try to set a read lock and close file
        Slave0: close the file and remove it.
        """

        self.slaves[0].mark(self.id())
        self.slaves[0].mount(self.shares[0])
        file1 = self.slaves[0].open(self.filename, OpenMode.WRITE,
                                    CreationMode.CREATE)
        file1.setLock(Lock(0, 500, LockType.LOCKWRITE))

        self.slaves[1].mark(self.id())
        self.slaves[1].mount(self.shares[0])
        file2 = self.slaves[1].open(self.filename, OpenMode.READ,
                                    CreationMode.OPEN)
        self.assertRaises(RECT.Error, file2.setLock,
                          Lock(0, 500, LockType.LOCKREAD))

        file2.close()
        file1.close()
        self.slaves[0].unlink(self.filename)
        self.slaves[1].umount(self.shares[0])
        self.slaves[0].umount(self.shares[0])


if __name__ == '__main__':
    main()

